﻿using DesignForTesting;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Rhino.Mocks;
using System;

namespace TestsLibrary
{
    /// <summary>
    ///This is a test class for PersonReaderTest and is intended
    ///to contain all PersonReaderTest Unit Tests
    ///</summary>
    [TestClass()]
    public class PersonReaderTest
    {
        private TestContext testContextInstance;

        private MockRepository ContextMockRepository
        {
            get;
            set;
        }

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        [TestInitialize()]
        public void MyTestInitialize()
        {
            ContextMockRepository = new MockRepository();
        }
        
        //Use TestCleanup to run code after each test has run
        [TestCleanup()]
        public void MyTestCleanup()
        {
            ContextMockRepository.VerifyAll();
            ContextMockRepository = null;
        }
        
        #endregion


        /// <summary>
        ///A test for ReadNext
        ///</summary>
        [TestMethod()]
        public void ReadNextTest()
        {
            //Mock!
            IDataReader dataReader = ContextMockRepository.CreateMock<IDataReader>();
            Expect.Call(dataReader.ReadNext()).Return(true);
            //Expect.Call(dataReader.Count).Return(3);
            Expect.Call(dataReader[0]).Return("Gabriele");
            Expect.Call(dataReader[1]).Return("Zannoni");
            Expect.Call(dataReader[2]).Return("Ing.");
            Expect.Call(dataReader.ReadNext()).Return(false);
            ContextMockRepository.Replay(dataReader);

            PersonReader target = new PersonReader(dataReader); 
            Person actual;
            actual = target.ReadNext();
            Assert.AreEqual("Gabriele", actual.FirstName);
            Assert.AreEqual("Zannoni", actual.SecondName);
            Assert.AreEqual("Ing.", actual.Title);
            actual = target.ReadNext();
            Assert.IsNull(actual);
        }

        /// <summary>
        ///A test for PersonReader Constructor
        ///</summary>
        [TestMethod()]
        [ExpectedException(typeof(ArgumentNullException))]
        public void PersonReaderConstructorTest()
        {
            IDataReader dataReader = null; 
            PersonReader target = new PersonReader(dataReader);
        }
    }
}
